/**
 * Copyright (C) 2003-2007 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
/**
 * Copyright (C) 2003-2007 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the Honest Public License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Honest Public License for more details.
 *
 * You should have received a copy of the Honest Public License
 * along with this program; if not, write to Funambol,
 * 643 Bair Island Road, Suite 305 - Redwood City, CA 94063, USA
 */
package com.funambol.framework.core;

/**
 * Corresponds to &lt;Alert&gt; element in SyncML represent DTD
 *
 * @see AlertCode
 *
 * @version $Id: Alert.java,v 1.5 2007/01/11 11:48:49 nichele Exp $
 */
public class Alert
extends ItemizedCommand
implements java.io.Serializable {

    // --------------------------------------------------------------- Constants
    public static String COMMAND_NAME = "Alert";

    // ------------------------------------------------------------ Private data
    private int data;
    private String correlator;
    // ------------------------------------------------------------ Constructors

    /** For serialization purposes */
    public Alert() {}

    /**
     * Creates a new Alert object with the given command identifier,
     * noResponse, authentication credential, alert code, the link correlator
     * and array of item
     *
     * @param cmdID  the command identifier - NOT NULL
     * @param noResp is true if no response is required
     * @param cred   the authentication credential
     * @param data   the code of Alert
     * @param correlator the link between the command and an asynchronous response
     * @param items the array of item - NOT NULL
     *
     */
    public Alert(final CmdID   cmdID     ,
                 final boolean noResp    ,
                 final Cred    cred      ,
                 final int     data      ,
                 final String  correlator,
                 final Item[]  items     ) {

        super(cmdID, items);

        this.noResp  = (noResp) ? new Boolean(noResp) : null;
        setCred(cred);
        this.data       = data      ;
        this.correlator = correlator;
    }

    /**
     * Creates a new Alert object with the given command identifier,
     * noResponse, authentication credential, alert code and array of item
     *
     * @param cmdID command identifier - NOT NULL
     * @param noResp is true if no response is required
     * @param cred the authentication credential
     * @param data the code of Alert
     * @param items the array of item - NOT NULL
     *
     */
    public Alert(final CmdID   cmdID ,
                 final boolean noResp,
                 final Cred    cred  ,
                 final int     data  ,
                 final Item[]  items ) {
        this(cmdID, noResp, cred, data, null, items);
    }

    // ---------------------------------------------------------- Public methods

    /**
     * Gets the alert code
     *
     * @return the alert code
     */
    public int getData() {
        return data;
    }

    /**
     * Sets the alert code
     *
     * @param data the alert code
     */
    public void setData(int data) {
        this.data = data;
    }

    /**
     * Gets the correlator property
     *
     * @return the correlator property
     */
    public String getCorrelator() {
        return correlator;
    }

    /**
     * Sets the link between the command and an asynchronous response
     *
     * @param correlator the link
     */
    public void setCorrelator(String correlator) {
        this.correlator = correlator;
    }

    /**
     * Gets the command name property
     *
     * @return the command name property
     */
    public String getName() {
        return Alert.COMMAND_NAME;
    }
}
