/*
 * Copyright (C) 2003-2007 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * ---
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the Honest Public License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Honest Public License for more details.
 *
 * You should have received a copy of the Honest Public License
 * along with this program; if not, write to Funambol,
 * 643 Bair Island Road, Suite 305 - Redwood City, CA 94063, USA
 */

package com.funambol.framework.core;

import java.util.*;

/**
 * This class represents the content type property such as property name, value,
 * display name, size and the content type parameters
 *
 *  @version $Id: CTProperty.java,v 1.2 2007/01/11 11:48:49 nichele Exp $
 */
public class CTProperty 
extends CTParameter
implements java.io.Serializable {

    // ------------------------------------------------------------ Private data
    
    private ArrayList parameters;

    // ------------------------------------------------------------ Constructors

    /** For serialization purposes */
    protected CTProperty() {
        super();
        parameters = new ArrayList();
    }
    
    /**
     * Creates a new PropParam object given its name
     *
     * @param name - the PropParam name
     */
    public CTProperty(String name) {
        super(name);
    }

    /**
     * Creates a new ContentTypeProperty object with the given name, value and
     * display name
     *
     * @param propName corresponds to &lt;PropName&gt; element in the SyncML
     *                  specification - NOT NULL
     * @param valEnum   corresponds to &lt;ValEnum&gt; element in the SyncML
     *                  specification
     * @param displayName corresponds to &lt;DisplayName&gt; element in the SyncML
     *                  specification
     * @param ctParameters the array of content type parameters - NOT NULL
     *
     */
    public CTProperty(final String propName,
                      final String[] valEnum,
                      final String displayName,
                      final CTParameter[] parameters) {
        super(propName, valEnum, displayName);
        
        setParameters(parameters);
    }


    /**
     * Creates a new ContentTypeProperty object with the given name, value and
     * display name
     *
     * @param propName corresponds to &lt;PropName&gt; element in the SyncML
     *                  specification - NOT NULL
     * @param dataType corresponds to &lt;DataType&gt; element in the SyncML
     *                  specification
     * @param size corresponds to &lt;Size&gt; element in the SyncML
     *                  specification
     * @param displayName corresponds to &lt;DisplayName&gt; element in the SyncML
     *                  specification
     * @param ctParameters the array of content type parameters - NOT NULL
     *
     */
    public CTProperty(final String propName,
                      final String dataType,
                      final Long size,
                      final String displayName,
                      final CTParameter[] parameters) {

        super(propName, dataType, size, displayName);
        
        setParameters(parameters);
    }

    // ---------------------------------------------------------- Public methods

    /**
     * Gets the array of ContentTypeParameter
     *
     * @return the size propeties
     */
    public ArrayList getParameters() {
        return parameters;
    }

    /**
     * Sets an array of content type properties
     *
     * @param ctParameters array of content type properties
     *
     */
    public void setParameters(CTParameter[] parameters) {
        setParameters((parameters != null) ? Arrays.asList(parameters) : null);
    }

    /**
     * Sets an array of content type properties
     *
     * @param ctParameters array of content type properties
     *
     */
    public void setParameters(List parameters) {
        if (parameters != null) {
            if (this.parameters == null) {
                this.parameters = new ArrayList();
            } else {
                this.parameters.clear();
            }
            this.parameters.addAll(parameters);
        } else {
            this.parameters = null;
        }
    }
    
   
}
