/*
 * Copyright (C) 2003-2007 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * ---
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the Honest Public License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Honest Public License for more details.
 *
 * You should have received a copy of the Honest Public License
 * along with this program; if not, write to Funambol,
 * 643 Bair Island Road, Suite 305 - Redwood City, CA 94063, USA
 */
package com.funambol.framework.core;

import java.util.*;

/**
 * This class contains the information on CTCap for DevInf v1.1.x
 *
 *
 * @version $Id: CTType.java,v 1.2 2007/01/11 11:48:49 nichele Exp $
 */
public class CTType implements java.io.Serializable {

    // ------------------------------------------------------------ Private data
    private String ctType;
    private ArrayList ctProperties = new ArrayList();

    // ------------------------------------------------------------ Constructors
    /** For serialization purposes */
    protected CTType() {}

    /**
     * Creates a new CTType object with the given information
     *
     * @param ctType an String CTType - NOT NULL
     * @param ctPropParams the array of content type properties and/or content
     *                     content type parameters - NOT NULL
     *
     */
    public CTType(final String ctType,
                  final CTProperty[] properties ) {
        setCTType(ctType);
        setCTProperties(properties);
    }
    
    /**
     * Creates a new CTType object with the given information
     *
     * @param ctType an String CTType - NOT NULL
     * @param ctPropParams the array of content type properties and/or content
     *                     content type parameters - NOT NULL
     *
     */
    public CTType(final String ctType,
                  final ArrayList ctProperties ) {
        setCTType(ctType);
        setCTPropParams(ctProperties);
    }

    // ---------------------------------------------------------- Public methods

    /**
     * Get a CTType String
     *
     * @return a CTType String
     */
    public String getCTType() {
        return this.ctType;
    }

    /**
     * Sets a CTType object
     *
     * @param ctType a CTType object
     */
    public void setCTType(String ctType) {
        if (ctType == null || ctType.length() == 0) {
            throw new IllegalArgumentException("ctType cannot be null");
        }
        this.ctType = ctType;
    }

    /**
     * Gets an array of content type properties and parameters
     *
     * @return an array of content type properties and parameters
     *
     */
    public ArrayList getCTProperties() {
        return this.ctProperties;
    }

    /**
     * Sets an array of content type properties and parameters
     *
     * @param ctPropParams array of content type properties and parameters
     *
     */
    public void setCTProperties(CTProperty[] ctProperties) {
        if (ctProperties == null) {
            throw new IllegalArgumentException("ctPropParams cannot be null");
        }
        this.ctProperties.clear();
        this.ctProperties.addAll(Arrays.asList(ctProperties));
    }

    /**
     * Sets an array of content type properties and parameters
     *
     * @param ctPropParams array of content type properties and parameters
     *
     * @deprecated use setCTProperties() instead
     *
     */
    public void setCTPropParams(ArrayList ctProperties) {
        if (ctProperties == null) {
            throw new IllegalArgumentException("ctPropParams cannot be null");
        }
        this.ctProperties.clear();
        this.ctProperties.addAll(ctProperties);
    }
}
