/**
 * Copyright (C) 2003-2007 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
/**
 * Copyright (C) 2003-2007 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the Honest Public License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Honest Public License for more details.
 *
 * You should have received a copy of the Honest Public License
 * along with this program; if not, write to Funambol,
 * 643 Bair Island Road, Suite 305 - Redwood City, CA 94063, USA
 */


package com.funambol.framework.core;

import java.util.*;

/**
 * Corresponds to the &lt;Ext&gt; element in the SyncML devinf DTD and specifies
 * the non-standard, experimental extensions supported by the device.
 *
 * @version $Id: Ext.java,v 1.5 2007/01/11 11:48:50 nichele Exp $
 */
public class Ext
implements java.io.Serializable {

    // ------------------------------------------------------------ Private data
    private String xNam;
    private ArrayList xVal = new ArrayList();

    // ------------------------------------------------------------ Constructors
    /**
     * In order to expose the server configuration like WS this constructor
     * must be public
     */
    public Ext() {}

    /**
     * Creates a new Ext object with the given name and value
     *
     *  @param xNam corresponds to the &lt;XNam&gt; element - NOT NULL
     *  @param xVal an array of parameters that corresponds to the &ltXVal&gt
     *                  element
     *
     */
    public Ext(final String xNam, final String[] xVal) {
        setXNam(xNam);
        setXVal(xVal);
    }

    // ---------------------------------------------------------- Public methods
    /**
     * Gets the name of the extension
     *
     * @return the name of the extension
     */
    public String getXNam() {
        return xNam;
    }

    /**
     * Sets the name of extension
     *
     * @param xNam the name of extension
     *
     */
    public void setXNam(String xNam) {
        if (xNam == null) {
            throw new IllegalArgumentException("XNam cannot be null");
        }
        this.xNam = xNam;
    }

    /**
     * Gets an array of extension values, if exist
     *
     * @return an array of extension values, if exist
     */
    public ArrayList getXVal() {
        return xVal;
    }

    /**
     * Sets the array of extension value
     *
     * @param xVal the array of extension value
     *
     */
    public void setXVal(String[] xVal) {
        if (xVal == null) {
            throw new IllegalArgumentException("xVal cannot be null");
        }
        this.xVal.clear();
        this.xVal.addAll(Arrays.asList(xVal));
    }

    /**
     * Sets the array of extension value
     *
     * @param xVal the array of extension value
     *
     */
    public void setXVal(ArrayList xVal) {
        if (xVal == null) {
            throw new IllegalArgumentException("xVal cannot be null");
        }
        this.xVal.clear();
        this.xVal.addAll(xVal);
    }
}
