/**
 * Copyright (C) 2003-2007 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
/**
 * Copyright (C) 2003-2007 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the Honest Public License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Honest Public License for more details.
 *
 * You should have received a copy of the Honest Public License
 * along with this program; if not, write to Funambol,
 * 643 Bair Island Road, Suite 305 - Redwood City, CA 94063, USA
 */


package com.funambol.framework.core;

/**
 * This class represents to the &lt;Source&gt; s defined by the SyncML
 * representation specifications
 *
 * @version $Id: Source.java,v 1.6 2007/01/11 11:48:50 nichele Exp $
 */
public class Source
implements java.io.Serializable {
    // ------------------------------------------------------------ Private data

    /**
     * Specifies the source specific URI
     */
    private String locURI;

    /**
     * Specifies the display name for the  source address
     */
    private String locName;

    // ------------------------------------------------------------ Constructors

    /**
     * For serialization purposes
     */
    public Source() {}

    /**
     * Creates a new Source object given its URI and display name.
     *
     * @param locURI the source URI - NOT NULL
     * @param locName the source display name - NULL ALLOWED
     *
     * @throws IllegalArgumentException if logURI is null.
     */
    public Source(final String locURI, final String locName) {
        setLocURI(locURI);
        this.locName = locName;
    }

    /**
     * Creates a new Source object given its URI
     *
     * @param locURI the source URI - NOT NULL
     *
     * @throws IllegalArgumentException if logURI is null.
     */
    public Source(final String locURI) {
        this(locURI, null);
    }

    // ------------------------------------------------------ Public methods

    /**
     * Returns the source URI value
     *
     * @return the source URI value
     */
    public String getLocURI() {
        return locURI;
    }

    /**
     * Sets the source URI
     *
     * @param locURI the source URI - NOT NULL
     *
     * @throws IllegalArgumentException if locURI is null
     */
    public void setLocURI(final String locURI) {
        if (locURI == null) {
            throw new IllegalArgumentException("locURI cannot be null");
        }
        this.locURI = locURI;
    }

    /**
     * Returns the source display name
     *
     * @return the source display name
     *
     */
    public String getLocName() {
        return locName;
    }

    /**
     * Sets the local name property
     *
     * @param locName the local name property
     *
     */
    public void setLocName(String locName) {
        this.locName = locName;
    }
}
