/**
 * Copyright (C) 2006-2007 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the Honest Public License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Honest Public License for more details.
 *
 * You should have received a copy of the Honest Public License
 * along with this program; if not, write to Funambol,
 * 643 Bair Island Road, Suite 305 - Redwood City, CA 94063, USA
 */
package com.funambol.framework.engine.transformer;


/**
 * This class represents a data encoder or decoder
 *
 * A DataTransformer is a component that given some data in some format
 * as input, it generates a transformation of the source data in output.
 * Examples of transformations are:
 *
 * - base64 encoding/decoding
 * - encryption/decription
 */
public abstract class DataTransformer {
    // ------------------------------------------------------------ Private data

    private String name;

    // ------------------------------------------------------------ Constructors


    /** Creates a new instance of DataTransformer */
    public DataTransformer(String name) {
        this.name = name;
    }

    // ---------------------------------------------------------- Public methods

    public String getName() {
        return name;
    }

    public void setName(String name) {
        this.name = name;
    }

    // -------------------------------------------------------- Abstract methdos

    public abstract byte[] transform(byte[] data, TransformationInfo info)
           throws TransformerException ;

}
