/**
 * Copyright (C) 2006-2007 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the Honest Public License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Honest Public License for more details.
 *
 * You should have received a copy of the Honest Public License
 * along with this program; if not, write to Funambol,
 * 643 Bair Island Road, Suite 305 - Redwood City, CA 94063, USA
 */

package com.funambol.framework.logging;

import java.util.HashMap;
import java.util.Map;

import org.apache.log4j.Appender;

/**
 * It's a cache of appender instances
 * @version $Id: AppendersCache.java,v 1.3 2007/01/11 11:48:50 nichele Exp $
 */
public class AppendersCache {

    // ------------------------------------------------------------ Private Data
    /**
     * The appenders cache
     */
    private static Map<String, Appender> cache = new HashMap();

    // ----------------------------------------------------------- Public method
    /**
     * Caches the given appender
     * @param appender the appender to cache
     */
    public static void cacheAppender(Appender appender) {
        if (appender == null) {
            return;
        }
        String name = appender.getName();
        if (name == null || name.length() == 0) {
            return ;
        }
        synchronized (cache) {
            cache.put(name, appender);
        }
    }

    /**
     * Returns, if found in cache, the appender with the given name
     * @param name the appender's name
     * @return the appender with the given name
     */
    public static Appender getAppender(String name) {
        Appender appender = null;
        synchronized (cache) {
            appender = cache.get(name);
        }
        return appender;
    }

    /**
     * Returns the available appenders
     * @return the available appenders
     */
    public static Map<String, Appender> getAppenders() {
        Map<String, Appender> cacheClone = null;
        synchronized (cache) {
            cacheClone = (Map)((HashMap)cache).clone();
        }
        return cacheClone;
    }

    /**
     * Clears the appenders cache
     */
    public static void clear() {
        synchronized (cache) {
            cache.clear();
        }
    }
}
