/**
 * Copyright (C) 2003-2007 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the Honest Public License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Honest Public License for more details.
 *
 * You should have received a copy of the Honest Public License
 * along with this program; if not, write to Funambol,
 * 643 Bair Island Road, Suite 305 - Redwood City, CA 94063, USA
 */
package com.funambol.framework.logging;

import java.util.logging.Logger;

/**
 * Provides a common logging facility for the framework.
 * <p>
 * The verbosity of the log can be set in the JDK logging properties files
 * giving one of the following values (case insensitively):
 * <ul>
 *   <li>SEVERE
 *   <li>WARNING
 *   <li>INFO
 *   <li>CONFIG
 *   <li>FINE
 *   <li>FINER
 *   <li>FINEST
 *   <li>ALL
 * </ul>
 *
 * For example, for the Funambol default logger, you can specify:
 * <pre>
 *   funambol.level=SEVERE
 * </pre>
 *
 * @version $Id
 */
public class Sync4jLogger {

    public static final String LOG_NAME = "funambol";

    /**
     * Returns the Funambol default logger
     *
     * @return the Funambol default logger
     */
    public static Logger getLogger() {
        return Logger.getLogger(LOG_NAME);
    }

    /**
     * Returns the logger associated with the name <i>LOG_NAME</i>.name.
     *
     * @param name the subname of the logger.
     *
     * @return the logger associated with the name <i>LOG_NAME</i>.name.
     */
    public static Logger getLogger(String name) {
        return Logger.getLogger(LOG_NAME + '.' + name);
    }
}
