/**
 * Copyright (C) 2005-2007 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the Honest Public License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Honest Public License for more details.
 *
 * You should have received a copy of the Honest Public License
 * along with this program; if not, write to Funambol,
 * 643 Bair Island Road, Suite 305 - Redwood City, CA 94063, USA
 */
package com.funambol.framework.notification;

import com.funambol.framework.core.Alert;

/**
 * This class is the principal component of the server alerted synchronization
 * architecture.
 * It is responsible for: creating the binary package and  sending the message
 * over the protocol and medium to be used.
 *
 * @version  $Id: NotificationEngine.java,v 1.3 2007/01/11 11:48:50 nichele Exp $
 */
public interface NotificationEngine {

    // ---------------------------------------------------------- Public Methods
    /**
     * Sends a notification message to the given device.
     *
     * @param deviceId device identifier
     * @param alerts   array of Alert command for the datastore to sync, used
     *                 to create the binary message to send
     * @param uimode   user intraction mode
     */
    void sendNotificationMessage(String  deviceId,
                                 Alert[] alerts  ,
                                 int     uimode  )
    throws NotificationException;

    /**
      * Sends a notification message to all devices of the principals with the
      * given username.
      *
      * @param username the username
      * @param alerts   array of Alert command for the datastore to sync, used
      *                 to create the binary message to send
      * @param uimode   user intraction mode
      */
    void sendNotificationMessages(String  username,
                                  Alert[] alerts  ,
                                  int     uimode  )
    throws NotificationException;
}
