/**
 * Copyright (C) 2005-2007 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the Honest Public License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Honest Public License for more details.
 *
 * You should have received a copy of the Honest Public License
 * along with this program; if not, write to Funambol,
 * 643 Bair Island Road, Suite 305 - Redwood City, CA 94063, USA
 */
package com.funambol.framework.notification.builder;

import java.io.IOException;
import java.io.OutputStream;

/**
 * Class used to write bit into OutputStream
 *
 * @version $Id: BitOutputStream.java,v 1.3 2007/01/11 11:48:50 nichele Exp $
 */
public class BitOutputStream {

    // ------------------------------------------------------------ Private data
    private OutputStream out;
    private int buffer;
    private int bitCount = 7;

    // ------------------------------------------------------------- Constructor
    public BitOutputStream(OutputStream out) {
        this.out = out;
    }

    // ----------------------------------------------------------- Public method
    synchronized public void writeBit(int bit) throws IOException {
        if (out == null)
            throw new IOException("Already closed");

        if (bit != 0 && bit != 1) {
            throw new IOException(bit + " is not a bit");
        }

        buffer |= bit << bitCount;

        bitCount--;

        if (bitCount == -1) {
            flush();
        }
    }

    private void flush() throws IOException {
        if (bitCount < 7) {
            out.write( (byte)buffer);
            bitCount = 7;
            buffer = 0;
        }
    }

    public void close() throws IOException {
        flush();
        out.close();
        out = null;
    }
}
