/**
 * Copyright (C) 2005-2007 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the Honest Public License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Honest Public License for more details.
 *
 * You should have received a copy of the Honest Public License
 * along with this program; if not, write to Funambol,
 * 643 Bair Island Road, Suite 305 - Redwood City, CA 94063, USA
 */
package com.funambol.framework.notification.builder;

import com.funambol.framework.core.Alert;
import com.funambol.framework.notification.Message;
import com.funambol.framework.notification.NotificationException;

/**
 * The implementations of this interface must build the notification message.
 *
 * @version $Id: DSNotificationMessageBuilder.java,v 1.4 2007/01/11 11:48:50 nichele Exp $
 */
public interface DSNotificationMessageBuilder {
        
    // ---------------------------------------------------------- Public Methods
    
    /**
     * Builds the notification message with the given information
     *
     * @param sessionId       the identifier of the session associated with the 
     *                        Notification Package
     * @param deviceId        the device id
     * @param deviceAddress   the address or phone number (depending on the 
     *                        implementation) of the device where the message 
     *                        should be sent
     * @param serverId        the id of the server
     * @param serverPw        server's password
     * @param serverNonce     server's nonce
     * @param alerts          array of Alert command for the datastore to sync, 
     *                        used to create the binary message to send
     * @param protocolVersion the version of SyncML protocol
     *
     * @return the message
     * @throws NotificationException
     */
    public Message buildMessage(int     sessionId      ,
                                String  deviceId       ,
                                String  deviceAddress  ,
                                String  serverId       ,
                                String  serverPw       ,
                                byte[]  serverNonce    ,
                                Alert[] alerts         ,
                                int     uimode         ,
                                float   protocolVersion)
    throws NotificationException;    
}