/**
 * Copyright (C) 2007 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the Honest Public License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Honest Public License for more details.
 *
 * You should have received a copy of the Honest Public License
 * along with this program; if not, write to Funambol,
 * 643 Bair Island Road, Suite 305 - Redwood City, CA 94063, USA
 */
package com.funambol.framework.security;

import com.funambol.framework.logging.FunambolLogger;
import com.funambol.framework.logging.FunambolLoggerFactory;
import java.security.Principal;

import com.funambol.framework.core.Cred;
import com.funambol.framework.server.Sync4jUser;

/**
 * Implements the basic functionalities of a <i>Officer</i>.
 *
 * @version $Id: AbstractOfficer.java,v 1.1 2007/02/21 14:30:14 luigiafassina Exp $
 */
public abstract class AbstractOfficer implements Officer, java.io.Serializable {

    // --------------------------------------------------------------- Constants
    protected static final String AUTH_B64_MD5 =
        Cred.AUTH_TYPE_BASIC + "," + Cred.AUTH_TYPE_MD5;

    protected FunambolLogger log =
        FunambolLoggerFactory.getLogger("funambol.auth");

    // ------------------------------------------------------------ Constructors
    /** Creates a new instance of AbstractOfficer */
    public AbstractOfficer() {
    }

    // ---------------------------------------------------------- Public methods
    /**
     * Which authetication types does the server support?
     * By default, the server supports both b64 and MD5.
     */
    private String clientAuth = AUTH_B64_MD5;

    /**
     * Returns the authentication type that the client must use
     *
     * @return the authentication type that the client must use
     */
    public String getClientAuth() {
        return this.clientAuth;
    }

    /**
     * Sets the authentication type that the client must use
     *
     * @param clientAuth the authentication type that the client must use
     */
    public void setClientAuth(String clientAuth) {
        this.clientAuth = clientAuth;
    }

    /**
     * Which type of authetication use the server to send his credential?
     */
    private String serverAuth = Cred.AUTH_NONE;

    /**
     * Returns the authentication type that the server must use
     *
     * @return the authentication type that the server must use
     */
    public String getServerAuth() {
        return this.serverAuth;
    }

    /**
     * Sets the authentication type that the server must use
     *
     * @param serverAuth the authentication type that the server must use
     */
    public void setServerAuth(String serverAuth) {
        this.serverAuth = serverAuth;
    }

    /**
     * Un-authenticates the given user.
     *
     * @param user the user to be unauthenticated
     *
     */
    public void unAuthenticate(Sync4jUser user) {
        //
        // Do nothing. In the current implementation, the authentication is
        // discarde as soon as the LoginContext is garbage collected.
        //
    }

    /**
     * Authorizes a resource.
     *
     * @param principal the requesting entity
     * @param resource the name (or the identifier) of the resource to be
     *                 authorized
     * @return an AuthStatus the authentication status
     */
    public Officer.AuthStatus authorize(Principal principal, String resource) {
        return AuthStatus.AUTHORIZED;
    }

    // -------------------------------------------------------- Abstract methods
    /**
     * Authenticates credentials user.
     *
     * @param credentials the credentials to be authenticated
     *
     * @return the Sync4jUser relative to the given credentials, null if no
     *         user is found (credentials not authenticated)
     */
    public abstract Sync4jUser authenticateUser(Cred credentials);

}
