/**
 * Copyright (C) 2003-2007 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the Honest Public License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Honest Public License for more details.
 *
 * You should have received a copy of the Honest Public License
 * along with this program; if not, write to Funambol,
 * 643 Bair Island Road, Suite 305 - Redwood City, CA 94063, USA
 */
package com.funambol.framework.security;

import java.security.Principal;

import com.funambol.framework.core.Cred;
import com.funambol.framework.server.Sync4jUser;

/**
 * This the interface that defines how credentials are authenticated to a device
 * and how resources are authorized for a given credentials.
 *
 * @version $Id: Officer.java,v 1.7 2007/01/27 15:52:23 nichele Exp $
 */
public interface Officer {

    /**
     * Status used by authorize method
     */
    public enum AuthStatus {
        RESOURCE_NOT_AVAILABLE,
        INVALID_RESOURCE,
        NOT_AUTHORIZED,
        AUTHORIZED,
        PAYMENT_REQUIRED
    }

    /**
     * Authenticates credentials user.
     *
     * @param credentials the credentials to be authenticated
     *
     * @return the Sync4jUser relative to the given credentials, null if no user is found
     *         (credentials not authenticated)
     */
    public Sync4jUser authenticateUser(Cred credentials);

    /**
     * Un-authenticates the given user.
     *
     * @param user the user to be unauthenticated
     */
    public void unAuthenticate(Sync4jUser user);

    /**
     * Authorizes a resource.
     *
     * @param principal the requesting entity
     * @param resource the name (or the identifier) of the resource to be authorized
     * @return an AuthStatus
     */
    public AuthStatus authorize(Principal principal, String resource);

    /**
     * Returns a comma separeted list of the officer supported authentication types
     *
     * @return a comma separeted list of the officer supported authentication types
     */
    public String getClientAuth();

    /**
     * Returns the authentication type that the server must use
     *
     * @return the authentication type that the server must use
     */
    public String getServerAuth();
}

