/**
 * Copyright (C) 2005-2007 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the Honest Public License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Honest Public License for more details.
 *
 * You should have received a copy of the Honest Public License
 * along with this program; if not, write to Funambol,
 * 643 Bair Island Road, Suite 305 - Redwood City, CA 94063, USA
 */
package com.funambol.framework.server;

import org.apache.commons.lang.builder.ToStringBuilder;

/**
 * Represent the mapping between the LUID from the client Items and
 * the GUID from the server Items.
 *
 * @version $Id: ClientMappingEntry.java,v 1.3 2007/01/11 11:48:51 nichele Exp $
 */
public class ClientMappingEntry {

    // -------------------------------------------------------------- Properties

    // The guid
    private String guid;

    /**
     * Getter for property guid
     * @return String
     */
    public String getGuid() {
        return guid;
    }

    /**
     * Setter for property guid
     * @param guid String
     */
    public void setGuid(String guid) {
        this.guid = guid;
    }

    // The luid
    private String luid;

    /**
     * Getter for property luid
     * @return String
     */
    public String getLuid() {
        return luid;
    }

    /**
     * Setter for property luid
     * @param guid String
     */
    public void setLuid(String luid) {
        this.luid = luid;
    }


    // The last anchor for this mapping
    private String lastAnchor;

    /**
     * Getter for property lastAnchor
     * @return String
     */
    public String getLastAnchor() {
        return lastAnchor;
    }

    /**
     * Setter for property lastAnchor
     * @param guid String
     */
    public void setLastAnchor(String lastAnchor) {
        this.lastAnchor = lastAnchor;
    }

    // ------------------------------------------------------------ Constructors
    /**
     * Creates a new ClientMappingEntry
     * @param guid String
     * @param luid String
     * @param lastAnchor String
     */
    public ClientMappingEntry(String guid, String luid, String lastAnchor) {
        this.guid       = guid;
        this.luid       = luid;
        this.lastAnchor = lastAnchor;
    }

    /**
     * Creates a new ClientMappingEntry
     * @param guid String
     */
    public ClientMappingEntry(String guid) {
        this.guid       = guid;
    }

    // ---------------------------------------------------------- Public Methods

    /**
     * Returns true is the last anchor is not null or empty
     * @return boolean
     */
    public boolean isValid() {
        if (lastAnchor == null || lastAnchor.equals("")) {
            return false;
        }
        return true;
    }
    /**
     * Two entry are equal if their guid are equal.
     *
     * @param o the object this instance must be compared to.
     *
     * @return the given object is equal to this object
     */
    public boolean equals(Object o) {
        if (!(o instanceof ClientMappingEntry)) return false;

        return ((ClientMappingEntry)o).getGuid().equals(getGuid());
    }

    /**
     * Returns the hash code of this object
     *
     * @return the hash code of this object
     */
    public int hashCode() {
        return getGuid().hashCode();
    }

    public String toString() {
         ToStringBuilder sb = new ToStringBuilder(this);

         sb.append("guid", guid);
         sb.append("luid", luid);

         return sb.toString();
    }


}
