/**
 * Copyright (C) 2003-2007 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the Honest Public License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Honest Public License for more details.
 *
 * You should have received a copy of the Honest Public License
 * along with this program; if not, write to Funambol,
 * 643 Bair Island Road, Suite 305 - Redwood City, CA 94063, USA
 */
package com.funambol.framework.server;

import java.io.Serializable;
import org.apache.commons.lang.builder.ToStringBuilder;

import com.funambol.framework.engine.source.SyncSource;
import com.funambol.framework.engine.source.SyncSourceErrorDescriptor;

/**
 * This class represents a Funambol source type with a list of SyncSource
 * with the same type.
 *
 * @version $Id: Sync4jSourceType.java,v 1.5 2007/01/11 11:48:51 nichele Exp $
 */
public class Sync4jSourceType implements Serializable {

    private String sourceTypeId;
    private String description;
    private String className;
    private String adminClass;
    private SyncSource[] syncSource;
    private SyncSourceErrorDescriptor[] syncSourceFailed;

    /** Creates a new instance of SyncSourceType */
    public Sync4jSourceType() {
        this(null, null, null,null);
    }

    public Sync4jSourceType(String sourceTypeId, 
                            String description , 
                            String className   ,
                            String adminClass  ) {
        this.sourceTypeId = sourceTypeId;
        this.description  = description ;
        this.className    = className   ;
        this.adminClass   = adminClass  ;
    }

    /** Getter for property sourceTypeId.
     * @return Value of property sourceTypeId.
     *
     */
    public String getSourceTypeId() {
        return sourceTypeId;
    }

    /** Setter for property sourceTypeId.
     * @param sourceTypeId New value of property sourceTypeId.
     *
     */
    public void setSourceTypeId(String sourceTypeId) {
        this.sourceTypeId = sourceTypeId;
    }

    /** Getter for property description.
     * @return Value of property description.
     *
     */
    public String getDescription() {
        return description;
    }

    /** Setter for property description.
     * @param description New value of property description.
     *
     */
    public void setDescription(String description) {
        this.description = description;
    }

    /** Getter for property className.
     * @return Value of property className.
     *
     */
    public String getClassName() {
        return className;
    }

    /** Setter for property className.
     * @param className New value of property className.
     *
     */
    public void setClassName(String className) {
        this.className = className;
    }

    /** Getter for property adminClass.
     * @return Value of property adminClass.
     *
     */
    public String getAdminClass() {
        return adminClass;
    }

    /** Setter for property adminClass.
     * @param adminClass New value of property adminClass.
     *
     */
    public void setAdminClass(String adminClass) {
        this.adminClass = adminClass;
    }

    /**
     * Set an array of instances of SyncSource
     */
    public void setSyncSources(SyncSource[] syncSource) {
        this.syncSource = syncSource;
    }
    public SyncSource[] getSyncSources() {
        return this.syncSource;
    }

    /**
     * Set an array of SyncSourceException with the uri of SyncSource with
     * creations failed
     */
    public void setSyncSourcesFailed(SyncSourceErrorDescriptor[] syncSourceFailed) {
        this.syncSourceFailed = syncSourceFailed;
    }
    public SyncSourceErrorDescriptor[] getSyncSourcesFailed() {
        return this.syncSourceFailed;
    }
    
    public String toString() {
        ToStringBuilder sb = new ToStringBuilder(this);

         sb.append("sourceTypeId", sourceTypeId);
         sb.append("description" , description);
         sb.append("className"   , className);
         sb.append("adminClass"  , adminClass);

         return sb.toString();
    }

}
