/*
 * Copyright (C) 2005-2007 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*
 * Copyright (C) 2005-2007 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the Honest Public License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Honest Public License for more details.
 *
 * You should have received a copy of the Honest Public License
 * along with this program; if not, write to Funambol,
 * 643 Bair Island Road, Suite 305 - Redwood City, CA 94063, USA
 */
package com.funambol.framework.tools;

import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;
import java.util.TimeZone;
import java.util.Enumeration;
import java.util.Iterator;
import java.util.Calendar;

import java.util.zip.CRC32;

import java.io.InputStream;
import java.io.ByteArrayInputStream;
import java.io.IOException;

import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.ParserConfigurationException;

import org.xml.sax.SAXException;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;


/**
 * @version $Id: SourceUtils.java,v 1.4 2007/03/16 10:38:23 mauro Exp $
 */
public class SourceUtils {

    //---------------------------------------------------------------- Constants
    public static final String ROOT_NAME = "__root__name__";

    public static final String XML_VERSION = "<?xml version=\"1.0\" encoding=\""
                                           + System.getProperty("file.encoding")
                                           + "\"?>" ;
    /*
    * Test main
    */
    public static void main(String args[]) throws Exception{

        SourceUtils s = new SourceUtils();
        String str = new String();
        str += "<contact><doc1>doc11</doc1><doc2>docu2</doc2></contact>";
        HashMap hash = new HashMap();
        hash = s.xmlToHashMap(str);
        String s2 = new String();
        s2 = s.hashMapToXml(hash);
        System.out.println(s2);

    }

    //----------------------------------------------------------- Public methods

   /**
     *
     * Make a HashMap of fieldName - fieldValue
     *
     * tagName   -> key
     * tagValue  -> value
     *
     * @param content
     * @return HashMap of fieldName - fieldValue
     * @throws Exception
     **/
    public static HashMap xmlToHashMap (String content)
    throws IOException, ParserConfigurationException, SAXException {
        DocumentBuilderFactory  docBuilderFactory  = null ;
        DocumentBuilder         docBuilder         = null ;
        Document                docXml             = null ;
        NodeList                lstChildren        = null ;
        Element                 el                 = null ;
        Node                    node               = null ;

        HashMap                 fields             = null ;
        String                  nodeValue          = null ;
        String                  rootName           = null ;

        InputStream             is                 = null ;


        content = dropTagCData(content);
        int p = content.indexOf("?>");
        if (p>0) {
            content = content.substring(p+2);
            content = XML_VERSION + content;
        }

        is = new ByteArrayInputStream(content.getBytes());

        docBuilderFactory = DocumentBuilderFactory.newInstance() ;
        docBuilder = docBuilderFactory.newDocumentBuilder()    ;
        docXml = docBuilder.parse(is)                          ;

        el = docXml.getDocumentElement()                       ;

        rootName = el.getTagName();

        lstChildren = el.getChildNodes()                       ;

        fields = new HashMap();

        for (int i=0, l = lstChildren.getLength(); i <  l; i++) {

            node = lstChildren.item(i);

            if (node.getNodeType() == org.w3c.dom.Node.ELEMENT_NODE) {

                if (node.getChildNodes().item(0) != null) {
                    nodeValue = node.getChildNodes().item(0).getNodeValue();
                } else {
                    nodeValue = "";
                }

                fields.put(node.getNodeName(), nodeValue);

            }

        }

        //
        // Put the rootName in the hashMap with key ROOT_NAME that is a
        // conventional name to store the name of the root
        //
        if (rootName != null) {
            fields.put(ROOT_NAME, rootName);
        }
        return fields;
    }

     /**
     * Make xml from an HashMap.
     * key   -> tagName
     * value -> tagValue
     *
     * @param fields
     * @return String with the xml
     * @throws Exception
     **/
    public static String hashMapToXml (Map fields) throws Exception {
        String fieldName  = null;
        String fieldValue = null;
        String rootName   = null;
        String message    = "";

        Iterator i = fields.keySet().iterator();;
        while(i.hasNext()) {

            fieldName  = (String) i.next()              ;
            fieldValue = StringTools.escapeBasicXml((String)fields.get((String)fieldName));

            if (fieldName.equals (ROOT_NAME)) {
                rootName = fieldValue;
            } else {
                message =  message  +
                       "<"             +
                       fieldName       +
                       ">"             +
                       fieldValue      +
                       "</"            +
                       fieldName       +
                       ">" ;
        }
        }

        // using CDATA
        // message = "<![CDATA[" + "<" + rootName + ">" + message;
        // message = message  + "</" + rootName + ">]]>"          ;

        // without CDATA
        message = "<" + rootName + ">" + message  ;
        message = message  + "</" + rootName + ">";

        return XML_VERSION + message;
    }

    //--------------------------------------------------------- Private methods

    /**
     * @param content
     * @return input string by drop CData tag
     **/
    private static String dropTagCData(String content) {

        int startData = 0;
        int endData   = 0;

        if (content.indexOf("<![CDATA[") != - 1) {

            startData = content.indexOf("<![CDATA[") + "<![CDATA[".length();

            //
            // for server bug: server create end CDATA with ]]]]>
            //
            endData   = content.lastIndexOf("]]]]>");

            if (endData == -1) {
               endData   = content.lastIndexOf("]]>");
            }

            content = content.substring(startData, endData);
        }
            return content;
        }

    /**
     * Returns the CRC value for the given byte[]
     *
     * @param value
     * @return CRC of value
     **/
    public static long computeCRC(byte[] value) {

        CRC32 checksumEngine = new CRC32();

        checksumEngine.update(value, 0, value.length);
        long checksumMessage = checksumEngine.getValue();
        checksumEngine.reset();

        return checksumMessage;
    }
}
