/*
 * Copyright (C) 2003-2007 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Copyright (C) 2003-2007 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the Honest Public License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Honest Public License for more details.
 *
 * You should have received a copy of the Honest Public License
 * along with this program; if not, write to Funambol,
 * 643 Bair Island Road, Suite 305 - Redwood City, CA 94063, USA
 */
package com.funambol.framework.tools;

import java.io.ByteArrayOutputStream;

import org.jibx.runtime.*;

import com.funambol.framework.core.SyncML;

/**
 * This class contains some helper methods to translate a tree of objects into
 * a SyncML message and to be used by JiBX.
 *
 * @version $Id: Util.java,v 1.1 2006/10/15 16:21:44 stefano_fornari Exp $
 */
public class SyncMLUtil {
    // ------------------------------------------------------------ Constructors

    // ---------------------------------------------------------- Public methods

    /**
     * Serialize Long value to string.
     *
     * @param value Long value to be serialized
     *
     * @return the representation of value
     */
    public static String serializeWrapLong(Long value) {
        return String.valueOf(value);
    }

    /**
     * Deserialize Long from string
     *
     * @param value string to be parsed
     *
     * @return the representation of value
     */
    public static Long deserializeWrapLong(String value) {
        if (value == null || value.equals("")) {
        return null;
        }
        return Long.valueOf(value.trim());
    }

    public static Boolean deserializeBoolean(String value) {
        if (value != null &&
        (value.equals("") || value.equalsIgnoreCase("true"))) {
            return Boolean.TRUE;
        }
        return null;
    }

    public static String serializeBoolean(Boolean value) {
        return value.booleanValue() ? "" : null;
    }

    /**
     * Use marshall to create the representation XML of the object SyncML
     *
     * @param syncML the object SyncML
     *
     * @return the representation XML of the message
     */
    public static String toXML(SyncML syncML) {
        String message = null;
        try {

            ByteArrayOutputStream bout = new ByteArrayOutputStream();

            IBindingFactory f = BindingDirectory.getFactory(SyncML.class);
            IMarshallingContext c = f.createMarshallingContext();
            c.setIndent(0);
            c.marshalDocument(syncML, "UTF-8", null, bout);

            message = new String(bout.toByteArray());

        } catch(Exception e) {
            e.printStackTrace();
        }
        return message;
    }

    /**
     * Use marshall to create the representation XML of the object,
     * therefore must exist the mapping in the binding file.
     *
     * @param obj the object
     *
     * @return the representation XML of the object
     */
    public static String toXML(Object obj) {
        String message = null;
        try {
            ByteArrayOutputStream bout = new ByteArrayOutputStream();

            IBindingFactory f = BindingDirectory.getFactory(obj.getClass());
            IMarshallingContext c = f.createMarshallingContext();
            c.setIndent(0);
            c.marshalDocument(obj, "UTF-8", null, bout);

            message = new String(bout.toByteArray());
        } catch(Exception e) {
            e.printStackTrace();
        }
        return message;
    }

}
