/*
 * Copyright (C) 2003-2007 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*
 * Copyright (C) 2003-2007 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the Honest Public License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Honest Public License for more details.
 *
 * You should have received a copy of the Honest Public License
 * along with this program; if not, write to Funambol,
 * 643 Bair Island Road, Suite 305 - Redwood City, CA 94063, USA
 */
package com.funambol.framework.tools.merge;

import java.util.Iterator;
import java.util.Map;
import java.util.HashMap;

/**
 * @version $Id: MergeResult.java,v 1.3 2007/01/18 17:20:48 nichele Exp $
 */
public class MergeResult {

    private Map propertiesA = null;
    private Map propertiesB = null;

    private boolean setA = false;
    private boolean setB = false;

    private String prefix = null;

    public MergeResult() {
        propertiesA = new HashMap();
        propertiesB = new HashMap();
    }

    public MergeResult(String prefix) {
        propertiesA = new HashMap();
        propertiesB = new HashMap();

        this.prefix = prefix;
    }

    public MergeResult(boolean setA, boolean setB) {

        this.setA = setA;
        this.setB = setB;

        propertiesA = new HashMap();
        propertiesB = new HashMap();
    }

    public boolean isSetARequired() {
        return (propertiesA.size() > 0) || setA;
    }

    public void addPropertyA(String prop) {
        propertiesA.put(prop, null);
    }

    public void addPropertyA(String prop, String text) {
        propertiesA.put(prop, text);
    }

    public void addPropertyB(String prop) {
        propertiesB.put(prop, null);
    }

    public void addPropertyB(String prop, String text) {
        propertiesB.put(prop, text);
    }

    public boolean isSetBRequired() {
        return (propertiesB.size() > 0) || setB;
    }

    public void addMergeResult(MergeResult otherResult) {
        addMergeResult(otherResult, otherResult.prefix);
    }

    public void addMergeResult(MergeResult otherResult, String newPrefix) {

        if (otherResult.prefix != null) {
            newPrefix = otherResult.prefix;
        }

        if (otherResult.setA) {
            setA = true;
        }
        if (otherResult.setB) {
            setB = true;
        }

        Iterator itA     = otherResult.propertiesA.keySet().iterator();
        StringBuffer sbA = null;
        String key       = null;
        while (itA.hasNext()) {
            key = (String)itA.next();
            sbA = new StringBuffer();
            sbA.append(newPrefix).append('/').append(key);
            addPropertyA(sbA.toString(), (String)otherResult.propertiesA.get(key));
        }

        Iterator itB     = otherResult.propertiesB.keySet().iterator();
        StringBuffer sbB = null;

        while (itB.hasNext()) {
            key = (String)itB.next();
            sbB = new StringBuffer();
            sbB.append(newPrefix).append('/').append(key);
            addPropertyB(sbB.toString(), (String)otherResult.propertiesB.get(key));
        }
    }

    public String toString() {
        StringBuffer sb = new StringBuffer();
        sb.append("<SetA: ").append(setA).append(", SetB: ").append(setB);
        sb.append(", [ListA: ");

        Iterator itA = propertiesA.keySet().iterator();
        int cont     = 0;
        String key   = null;
        String value = null;

        while (itA.hasNext()) {
            key = (String)itA.next();
            value = (String)propertiesA.get(key);
            if (cont != 0) {
                sb.append(", ");
            }
            sb.append(prefix).append('/').append(key);
            sb.append( "(").append(value).append(")");
            cont++;
        }
        sb.append("]");

        sb.append(", [ListB: ");

        Iterator itB = propertiesB.keySet().iterator();
        cont = 0;
        while (itB.hasNext()) {
            key = (String)itB.next();
            value = (String)propertiesB.get(key);

            if (cont != 0) {
                sb.append(", ");
            }
            sb.append(prefix).append('/').append(key);
            sb.append( "(").append(value).append(")");
            cont++;
        }
        sb.append("]>");

        return sb.toString();
    }
    
    
    
}
