/*
 * Copyright (C) 2003-2007 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*
 * Copyright (C) 2003-2007 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the Honest Public License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Honest Public License for more details.
 *
 * You should have received a copy of the Honest Public License
 * along with this program; if not, write to Funambol,
 * 643 Bair Island Road, Suite 305 - Redwood City, CA 94063, USA
 */
package com.funambol.framework.tools.merge;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.apache.commons.lang.StringUtils;

/**
 * @version $Id: MergeUtils.java,v 1.3 2007/01/18 17:20:48 nichele Exp $
 */
public class MergeUtils {

    
    /**
     *
     */
    public static MergeResult compareStrings(String stringA, String stringB) {

        MergeResult result =  new MergeResult(false, false);

        if (stringA == null && stringB == null) {
            result = new MergeResult(false, false);
        }

        if (stringA != null && stringB != null) {

            if (stringA.trim().equals(stringB.trim())) {
                result = new MergeResult(false, false);
                return result;
            }

            if (stringA.trim().length() > 0) {
                result = new MergeResult(false, true);
                result.addPropertyB("string", stringA + ", " + stringB);
                return result;
            }

            if (stringB.trim().length() > 0) {
                result = new MergeResult(true, false);
                result.addPropertyA("string", stringA + ", " + stringB);
                return result;
            }

            //
            // This must not hap
            //
            result = new MergeResult(false, false);
        }

        if (stringA == null && stringB != null) {

            if (stringB.trim().length() > 0) {
                result = new MergeResult(true, false);
                result.addPropertyA("string", "null, " + stringB);
            } else {
                result = new MergeResult(false, false);
            }

        }

        if (stringA != null && stringB == null) {

            if (stringA.trim().length() > 0) {
                result = new MergeResult(false, true);
                result.addPropertyB("string", stringA + ", null");
            } else {
                result = new MergeResult(false, false);
            }
        }

        return result;
    }


    /**
     *
     */
    public static MergeResult compareIntegers(Integer integerA, Integer integerB) {

        MergeResult result = null;

        if (integerA == null && integerB == null) {
            return  new MergeResult(false, false);
        }

        if (integerA != null && integerB == null) {
            result = new MergeResult();
            result.addPropertyB("Integer", integerA.intValue() + ", null");
            return result;
        }

        if (integerA == null && integerB != null) {
            result = new MergeResult();
            result.addPropertyA("Integer", "null, " + integerB.intValue());
            return result;
        }

        if (integerA.compareTo(integerB) != 0) {
            //
            // The values are different so
            // an update on integerB is required
            //
            result = new MergeResult();
            result.addPropertyB("Integer", integerA.intValue() + ", " + integerB.intValue());
            return result;
        }

        return new MergeResult(false, false);
    }


    /**
     *
     */
    public static MergeResult compareShorts(Short shortA, Short shortB) {

        MergeResult result = null;

        if (shortA == null && shortB == null) {
            return  new MergeResult(false, false);
        }

        if (shortA != null && shortB == null) {
            result = new MergeResult();
            result.addPropertyB("Short", shortA.shortValue() + ", null");
            return result;
        }

        if (shortA == null && shortB != null) {
            result = new MergeResult();
            result.addPropertyA("Short", "null, " + shortB.shortValue());
            return result;
        }

        if (shortA.compareTo(shortB) != 0) {
            //
            // The values are different so
            // an update on shortB is required
            //
            result = new MergeResult();
            result.addPropertyB("Short", shortA.shortValue() + ", " + shortB.shortValue());
            return result;
        }

        return new MergeResult(false, false);
    }


    /**
     *
     */
    public static MergeResult compareBoolean(Boolean booleanA, Boolean booleanB) {

        MergeResult result = null;

        if (booleanA == null && booleanB == null) {
            return  new MergeResult(false, false);
        }

        if (booleanA != null && booleanB == null) {
            result = new MergeResult();
            result.addPropertyB("Boolean", booleanA.booleanValue() + ", null");
            return result;
        }

        if (booleanA == null && booleanB != null) {
            result = new MergeResult();
            result.addPropertyA("Boolean", "null, " + booleanB.booleanValue());
            return result;
        }

        if (booleanA.booleanValue() != booleanB.booleanValue()) {
            //
            // The values are different so
            // an update on booleanB is required
            //
            result = new MergeResult();
            result.addPropertyB("Boolean",
                                booleanA.booleanValue() + ", " + booleanB.booleanValue());
            return result;
        }

        return new MergeResult(false, false);
    }


    /**
     *
     */
    public static MergeResult mergeMap(Map mapA, Map mapB) {

        MergeResult result = new MergeResult();

        if (mapA == null && mapB != null) {
            mapA = new HashMap();
            mapA.putAll(mapB);
            result.addPropertyA("MapA", "MapA is null, MapB isn't null");
            return result;
        }

        if (mapA != null && mapB == null) {
            mapB = new HashMap();
            mapB.putAll(mapA);
            result.addPropertyB("MapB", "MapA isn't null, MapB is null");
            return result;
        }

        if (mapA == null && mapB == null) {
            return result;
        }

        Iterator itA          = mapA.keySet().iterator();
        String key            = null;
        String valueA         = null;
        String valueB         = null;
        MergeResult tmpResult = null;

        while (itA.hasNext()) {
            key = (String) itA.next();
            valueA = (String) mapA.get(key);
            valueB = (String) mapB.get(key);

            tmpResult = MergeUtils.compareStrings(valueA, valueB);
            if (tmpResult.isSetARequired()) {
                mapA.put(key, valueB);
            } else if (tmpResult.isSetBRequired()) {
                mapB.put(key, valueA);
            }

            result.addMergeResult(tmpResult, key);
        }

        Iterator itB          = mapB.keySet().iterator();

        while (itB.hasNext()) {
            key = (String) itB.next();
            valueA = (String) mapA.get(key);

            if (valueA != null) {
                //
                // The property is already in MapA so we can continue
                // (we already have handled this prop in the previous cycle)
                //
                continue;
            }

            valueB = (String) mapB.get(key);

            tmpResult = MergeUtils.compareStrings(valueA, valueB);
            if (tmpResult.isSetARequired()) {
                mapA.put(key, valueB);
            } else if (tmpResult.isSetBRequired()) {
                mapB.put(key, valueA);
            }

            result.addMergeResult(tmpResult, key);
        }

        return result;
    }
    
}
