/**
 * Copyright (C) 2003-2007 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the Honest Public License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Honest Public License for more details.
 *
 * You should have received a copy of the Honest Public License
 * along with this program; if not, write to Funambol,
 * 643 Bair Island Road, Suite 305 - Redwood City, CA 94063, USA
 */
package com.funambol.server.admin.ejb;

import java.util.Map;

import javax.ejb.EJBLocalObject;

import org.apache.log4j.Appender;

import com.funambol.framework.config.LoggerConfiguration;

import com.funambol.framework.config.LoggingConfiguration;
import com.funambol.framework.core.Alert;
import com.funambol.framework.engine.source.SyncSource;
import com.funambol.framework.filter.Clause;
import com.funambol.framework.security.Sync4jPrincipal;
import com.funambol.framework.server.Capabilities;
import com.funambol.framework.server.LastTimestamp;
import com.funambol.framework.server.Sync4jDevice;
import com.funambol.framework.server.Sync4jModule;
import com.funambol.framework.server.Sync4jSource;
import com.funambol.framework.server.Sync4jUser;
import com.funambol.framework.server.error.ServerException;

import com.funambol.server.admin.AdminException;
import com.funambol.server.config.Configuration;
import com.funambol.server.config.ServerConfiguration;

/**
 * @version $Id: AdminLocal.java,v 1.9 2007/02/25 18:49:34 nichele Exp $
 */
public interface AdminLocal extends EJBLocalObject {

    /**
     * CONFIGURATION
     */
    public Configuration getConfig();

    /**
     * USER
     */
    public String[] getRoles()
    throws ServerException, AdminException;

    public Sync4jUser[] getUsers(Clause clause)
    throws ServerException, AdminException;

    public void addUser(Sync4jUser u)
    throws ServerException, AdminException;

    public void setUser(Sync4jUser u)
    throws ServerException, AdminException;

    public void deleteUser(String userName)
    throws ServerException, AdminException;

    public int countUsers(Clause clause)
    throws ServerException, AdminException;


    /**
     * DEVICE
     */
    public Sync4jDevice[] getDevices(Clause clause)
    throws ServerException, AdminException;

    public Sync4jDevice getDevice(String deviceId)
    throws ServerException, AdminException;

    public String addDevice(Sync4jDevice d)
    throws ServerException, AdminException;

    public void setDevice(Sync4jDevice d)
    throws ServerException, AdminException;

    public void deleteDevice(String deviceId)
    throws ServerException, AdminException;

    public int countDevices(Clause clause)
    throws ServerException, AdminException;

    /**
     * CAPABILITIES
     */
    public Capabilities getDeviceCapabilities(String deviceId)
    throws ServerException, AdminException;

    public Long setDeviceCapabilities(Capabilities caps, String deviceId)
    throws ServerException, AdminException;


    /**
     * PRINCIPAL
     */
    public Sync4jPrincipal[] getPrincipals(Clause clause)
    throws ServerException, AdminException;

    public long addPrincipal(Sync4jPrincipal p)
    throws ServerException, AdminException;

    public void deletePrincipal(long principalId)
    throws ServerException, AdminException;

    public int countPrincipals(Clause clause)
    throws ServerException, AdminException;

    /**
     * LAST TIMESTAMP
     */
    public LastTimestamp[] getLastTimestamps(Clause clauses)
    throws ServerException, AdminException;

    public void deleteLastTimestamp(long principalId, String sourceId)
    throws ServerException, AdminException;

    public int countLastTimestamps(Clause clauses)
    throws ServerException, AdminException;

    /**
     * MODULE
     */
    public Sync4jModule[] getModulesName()
    throws ServerException, AdminException;

    public Sync4jModule getModule(String moduleId)
    throws ServerException, AdminException;

    /**
     * SOURCE
     */
    public void addSource(String moduleId, String connectorId, String sourceTypeId, SyncSource source)
    throws ServerException, AdminException;

    public void setSource(String moduleId, String connectorId, String sourceTypeId, SyncSource source)
    throws ServerException, AdminException;

    public Sync4jSource[] getSync4jSources(Clause clauses)
    throws ServerException, AdminException;

    public void deleteSource(String sourceUri)
    throws ServerException, AdminException;

    public String[] getSyncSourceClasses(String[] sourceTypesId)
    throws ServerException, AdminException;


    /**
     * LOGGERS
     */
    public LoggerConfiguration[] getLoggers()
    throws ServerException, AdminException;

    public Map getAppenders()
    throws ServerException, AdminException;

    public void setAppender(Appender appender)
    throws ServerException, AdminException;

    public String getAppenderManagementPanel(String appenderClassName)
    throws ServerException, AdminException;

    public void setLoggerConfiguration(LoggerConfiguration loggerConfig)
    throws ServerException, AdminException;

    /**
     * SETTINGS
     */
    public ServerConfiguration getServerConfiguration()
    throws ServerException, AdminException;

    public void setServerConfiguration(ServerConfiguration serverConfig)
    throws ServerException, AdminException;

    public String getServerVersion() throws ServerException, AdminException;

    /**
     * SERVER BEANS
     */

    /**
     * Returns the server bean with the given name. If the bean does not exist,
     * am AdminException is thrown.
     *
     * @param bean the server bean
     *
     * @throws ServerException, AdminException
     */
    public Object getServerBean(String bean)
    throws ServerException, AdminException;

    /**
     * Sets the server bean with the given name.
     *
     * @param bean the server bean
     * @param obj the bean instance
     *
     * @throws ServerException, AdminException
     */
    public void setServerBean(String bean, Object obj)
    throws ServerException, AdminException;


    /**
     * NOTIFICATION ENGINE
     */
     
    /**     
     * Sends a notification message to the given device.
     *
     * @param deviceId device identifier
     * @param alerts   array of Alert command for the datastore to sync, used
     *                 to create the binary message to send
     * @param uimode   user intraction mode
     */
    public void sendNotificationMessage(String  deviceId,
                                        Alert[] alerts  ,
                                        int     uimode  )
    throws ServerException;

    /**
     * Sends a notification message to all devices of the principals with the
     * given username.
     *
     * @param username the username
     * @param alerts   array of Alert command for the datastore to sync, used
     *                 to create the binary message to send
     * @param uimode   user intraction mode
     */
    public void sendNotificationMessages(String  username,
                                         Alert[] alerts  ,
                                         int     uimode  )
    throws ServerException;
}
