/**
 * Copyright (C) 2003-2007 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the Honest Public License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Honest Public License for more details.
 *
 * You should have received a copy of the Honest Public License
 * along with this program; if not, write to Funambol,
 * 643 Bair Island Road, Suite 305 - Redwood City, CA 94063, USA
 */


package com.funambol.server.admin.ws;

import java.io.*;
import java.lang.Class;
import java.lang.ClassLoader;

import javax.servlet.*;
import javax.servlet.http.*;

import com.funambol.framework.logging.FunambolLogger;
import com.funambol.framework.logging.FunambolLoggerFactory;

/**
 * This servlet allows the downloading of a class as found by the servlet's
 * class loader. It can be used to remotely download classes deployed on the
 * server.
 * <p>
 * It accepts the following parameters:
 * <table>
 * <tr>
 * <td>class</td>
 * <td>the class to download in the dotted or slashed form and optionally ended
 *     by .class</td>
 * </tr>
 * </table>
 *
 *
 *
 *  @version $Id: ClassDownloadServlet.java,v 1.6 2007/01/11 11:49:00 nichele Exp $
 *
 */
public final class ClassDownloadServlet
extends javax.servlet.http.HttpServlet {

    // --------------------------------------------------------------- Constants

    public static final String LOG_NAME = "server";

    // ------------------------------------------------------------ Private data

    private FunambolLogger log = FunambolLoggerFactory.getLogger(LOG_NAME);

    // ---------------------------------------------------------- Public methods

    public void service(final HttpServletRequest  request,
                        final HttpServletResponse response)
    throws ServletException, IOException {
        if (log.isTraceEnabled()) {
            log.trace(request.toString());
            log.trace(request.getRequestURI());
        }

        String className = request.getPathInfo();

        if (className == null) {
            className = "";
        } else {
            //
            // skip the initial '/'
            //
            className = className.substring(1);
        }

        if (log.isTraceEnabled()) {
            log.trace("Requesting class: " + className);
        }

        //
        // First of all normalize the class name in order to be retrieved
        // by getResourceAsStream()
        //
        className = className.trim();
        if (className.endsWith(".class")) {
            int l = className.length();

            if (l>6) {
                className = className.substring(0, l-6);
            } else {
                className = "";
            }
        }
        className = className.replace('.', '/');
        className = className + ".class";

        if (log.isTraceEnabled()) {
            log.trace("Looking for class: " + className);
        }

        ClassLoader cl = this.getClass().getClassLoader();

        InputStream is = cl.getResourceAsStream(className);

        if (is == null) {
            if (log.isTraceEnabled()) {
                log.trace("Class '" + className + "' not found");
            }
            //
            // Class not found!
            //
            response.sendError(response.SC_NOT_FOUND, className + " not found");
            return;
        } else {
            if (log.isTraceEnabled()) {
                log.trace("Class '" + className + "' found");
            }
        }

        byte[] buf = new byte[4096];
        int i = 0;
        OutputStream os = null;
        try {
            os = response.getOutputStream();

            while ((i = is.read(buf)) > 0) {
                os.write(buf, 0, i);
            }
        } catch (IOException e) {
            log.error("Error processing the request", e);
        } finally {
            if (os != null) {
                os.close();
            }
            is.close();
        }
    }

    // --------------------------------------------------------- Private methods
}


