/**
 * Copyright (C) 2003-2007 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the Honest Public License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Honest Public License for more details.
 *
 * You should have received a copy of the Honest Public License
 * along with this program; if not, write to Funambol,
 * 643 Bair Island Road, Suite 305 - Redwood City, CA 94063, USA
 */

package com.funambol.server.config;

/**
 * This class is just a container of SyncServer engine configuration.
 *
 * @version $Id: EngineConfiguration.java,v 1.10 2007/01/11 11:49:00 nichele Exp $
 */
public class EngineConfiguration {

    // --------------------------------------------------------------- Constants
    public static final String DEFAULT_OFFICER =
            "com/funambol/server/security/UserProvisioningOfficer.xml";

    public static final String DEFAULT_INVENTORY =
            "com/funambol/server/inventory/PSDeviceInventory.xml";

    public static final String DEFAULT_STRATEGY =
            "com/funambol/server/engine/Strategy.xml";

    public static final String DEFAULT_STORE =
            "com/funambol/server/store/PersistentStoreManager.xml";

    public static final String DEFAULT_PIPELINE_MANAGER =
            "com/funambol/server/engine/pipeline/PipelineManager.xml";

    public static final String DEFAULT_USER_MANAGER =
            "com/funambol/server/admin/DBUserManager.xml";

    public static final String DEFAULT_DATA_TRANSFORMER_MANAGER =
            "com/funambol/server/engine/transformer/DataTransformerManager.xml";

    public static final String DEFAULT_LOGGING_CONFIGURATION =
            "com/funambol/server/logging/Logging.xml";

    public static final String DEFAULT_SESSION_HANDLER =
            "com.funambol.server.session.SyncSessionHandler";

    // ------------------------------------------------------------ Private data

    /**
     * The server URI
     */
    private String serverURI;

    /**
     * The SyncStrategy implementation to use
     */
    private String strategy = DEFAULT_STRATEGY;

    /**
     * The SessionHandler to use
     */
    private String sessionHandler = DEFAULT_SESSION_HANDLER;

    /**
     * The PersistentStore to use
     * @deprecated starting in v6, the stores are handled in separeted files
     *             stored in com/funambol/server/store directory
     */
    private String store = DEFAULT_STORE;

    /**
     * The SecurityOfficer to use
     */
    private String officer = DEFAULT_OFFICER;

    /**
     * The PipelineManager to use
     * @deprecated starting in v6, the synclets are handled in separeted files
     *             stored in com/funambol/server/engine/pipeline/input[output]
     *             directory
     */
    private String pipelineManager = DEFAULT_PIPELINE_MANAGER;

    /**
     * The UserManager to use
     */
    private String userManager = DEFAULT_USER_MANAGER;

    /**
     * The LoggingConfiguration to use
     * @deprecated starting in v6, the loggers are handled in separeted files
     *             stored in com/funambol/server/logger directory
     */
    private String loggingConfiguration = DEFAULT_LOGGING_CONFIGURATION;

    /**
     * The minimum MaxMsgSize allowed
     */
    private long minMaxMsgSize;

    /**
     * The DeviceInventory to use
     */
    private String deviceInventory = DEFAULT_INVENTORY;

    /**
     * The DataTransformerManager to use
     */
    private String dataTransformerManager = DEFAULT_DATA_TRANSFORMER_MANAGER;

    // ---------------------------------------------------------- Public methods

    /**
     * Getter for property serverURI.
     * @return Value of property serverURI.
     */
    public String getServerURI() {
        return serverURI;
    }

    /**
     * Setter for property serverURI.
     * @param serverURI New value of property serverURI.
     */
    public void setServerURI(String serverURI) {
        this.serverURI = serverURI;
    }

    /**
     * Getter for property strategy.
     * @return Value of property strategy.
     */
    public String getStrategy() {
        return strategy;
    }

    /**
     * Setter for property strategy.
     * @param strategy New value of property strategy.
     */
    public void setStrategy(String strategy) {
        this.strategy = strategy;
    }

    /**
     * Getter for property sessionHandler.
     * @return Value of property sessionHandler.
     */
    public String getSessionHandler() {
        return sessionHandler;
    }

    /**
     * Setter for property sessionHandler.
     * @param sessionHandler New value of property sessionHandler.
     */
    public void setSessionHandler(String sessionHandler) {
        this.sessionHandler = sessionHandler;
    }

    /**
     * Getter for property store.
     * @return Value of property store.
     * @deprecated starting in v6, the stores are handled in separeted files
     *             stored in com/funambol/server/store directory
     */
    public String getStore() {
        return store;
    }

    /**
     * Setter for property store.
     * @param store New value of property store.
     * @deprecated starting in v6, the stores are handled in separeted files
     *             stored in com/funambol/server/store directory
     */
    public void setStore(String store) {
        this.store = store;
    }

    /**
     * Getter for property officer.
     * @return Value of property officer.
     */
    public String getOfficer() {
        return officer;
    }

    /**
     * Setter for property officer.
     * @param officer New value of property officer.
     */
    public void setOfficer(String officer) {
        this.officer = officer;
    }

    /**
     * Getter for property pipelineManager.
     * @return Value of property pipelineManager.
     * @deprecated starting in v6, the synclets are handled in separeted files
     *             stored in com/funambol/server/engine/pipeline/input[output]
     *             directory
     */
    public String getPipelineManager() {
        return pipelineManager;
    }

    /**
     * Setter for property pipelineManager.
     * @param pipelineManager New value of property pipelineManager.
     * @deprecated starting in v6, the synclets are handled in separeted files
     *             stored in com/funambol/server/engine/pipeline/input[output]
     *             directory
     */
    public void setPipelineManager(String pipelineManager) {
        this.pipelineManager = pipelineManager;
    }

    /**
     * Getter for property userManager.
     * @return Value of property userManager.
     */
    public String getUserManager() {
        return userManager;
    }

    /**
     * Setter for property userManager.
     * @param userManager New value of property userManager.
     */
    public void setUserManager(String userManager) {
        this.userManager = userManager;
    }

    /**
     * Getter for property loggingConfiguration.
     * @return Value of property loggingConfiguration.
     * @deprecated starting in v6, the loggers are handled in separeted files
     *             stored in com/funambol/server/logger directory
     */
    public String getLoggingConfiguration() {
        return loggingConfiguration;
    }

    /**
     * Setter for property loggingConfiguration.
     * @param loggingConfiguration the new value of property loggingConfiguration.
     * @deprecated starting in v6, the loggers are handled in separeted files
     *             stored in com/funambol/server/logger directory
     */
    public void setLoggingConfiguration(String loggingConfiguration) {
        this.loggingConfiguration = loggingConfiguration;
    }

    /**
     * Getter for property minMaxMsgSize.
     * @return Value of property minMaxMsgSize.
     */
    public long getMinMaxMsgSize() {
        return minMaxMsgSize;
    }

    /**
     * Setter for property minMaxMsgSize.
     * @param minMaxMsgSize New value of property minMaxMsgSize.
     */
    public void setMinMaxMsgSize(long minMaxMsgSize) {
        this.minMaxMsgSize = minMaxMsgSize;
    }


    /**
     * Getter for property deviceInventory.
     * @return Value of property deviceInventory.
     */
    public String getDeviceInventory() {
        return deviceInventory;
    }

    /**
     * Setter for property deviceInventory.
     * @param deviceInventory New value of property deviceInventory.
     */
    public void setDeviceInventory(String deviceInventory) {
        this.deviceInventory = deviceInventory;
    }

    /**
     * Getter for property dataTransformerManager.
     * @return Value of property dataTransformerManager.
     */
    public String getDataTransformerManager() {
        return dataTransformerManager;
    }

    /**
     * Setter for property dataTransformerManager.
     * @param dataTransformerManager New value of property dataTransformerManager.
     */
    public void setDataTransformerManager(String dataTransformerManager) {
        this.dataTransformerManager = dataTransformerManager;
    }


    /**
     * String representation for debug purposes
     *
     * @return the string representation of this object
     */
    public String toString() {
        StringBuffer buf = new StringBuffer(128);

        buf.append(getClass().getName())
           .append('@')
           .append(hashCode())
           .append("{\n")
           .append("serverURI: ").append(serverURI).append('\n')
           .append("minMaxMsgSize: ").append(minMaxMsgSize).append('\n')
           .append("strategy: ").append(strategy).append('\n')
           .append("sessionHandler: ").append(sessionHandler).append('\n')
           .append("store: ").append(store).append('\n')
           .append("userManager: ").append(userManager).append('\n')
           .append("officer: ").append(officer).append('\n')
           .append("pipelineManager: ").append(pipelineManager).append('\n')
           .append("dataTransformerManager: ").append(dataTransformerManager).append('\n')
           .append("deviceInventory: ").append(deviceInventory).append('\n')
           .append("loggingConfiguration: ").append(loggingConfiguration).append('\n')
           .append('}');

        return buf.toString();
    }
 }
