/**
 * Copyright (C) 2005-2007 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the Honest Public License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Honest Public License for more details.
 *
 * You should have received a copy of the Honest Public License
 * along with this program; if not, write to Funambol,
 * 643 Bair Island Road, Suite 305 - Redwood City, CA 94063, USA
 */
package com.funambol.server.notification.builder;

import com.funambol.framework.core.Alert;
import com.funambol.framework.notification.Message;
import com.funambol.framework.notification.NotificationException;
import com.funambol.framework.notification.builder.*;

/**
 * This class is an implementation of DSNotificationMessageBuilder
 *
 * @see DSNotificationMessageBuilder
 *
 * @version $Id: DSNotificationMessageBuilderImpl.java,v 1.4 2007/01/11 11:49:25 nichele Exp $
 */
public class DSNotificationMessageBuilderImpl
implements DSNotificationMessageBuilder {

    // -------------------------------------------------------------- Constructor
    public DSNotificationMessageBuilderImpl() {
    }

    // ---------------------------------------------------------- Public Methods
    /**
     * Builds the notification message with the given information
     *
     * @param sessionId       the identifier of the session associated with the
     *                        Notification Package
     * @param deviceId        the device id
     * @param deviceAddress   the address or phone number (depending on the
     *                        implementation) of the device where the message
     *                        should be sent
     * @param serverId        the id of the server
     * @param serverPw        server's password
     * @param serverNonce     server's nonce
     * @param alerts          array of Alert command for the datastore to sync,
     *                        used to create the binary message to send
     * @param protocolVersion the version of the SyncML protocol
     *
     * @return the notification message
     * @throws NotificationException
     */
    public Message buildMessage(int     sessionId      ,
                                String  deviceId       ,
                                String  deviceAddress  ,
                                String  serverId       ,
                                String  serverPw       ,
                                byte[]  serverNonce    ,
                                Alert[] alerts         ,
                                int     uimode         ,
                                float   protocolVersion)
    throws NotificationException {

        //message digest
        DigestNotificationMessage digestMessage = new DigestNotificationMessage();
        digestMessage.setNonce(serverNonce);
        digestMessage.setServerId(serverId);
        digestMessage.setServerPw(serverPw);

        //message header
        HeaderMessage header = new HeaderMessage();
        header.setInitiator(HeaderMessage.INITIATOR_SERVER);

        header.setUiMode(uimode);

        header.setVersion(protocolVersion);
        header.setServerIdentifier(serverId);
        header.setSessionId(sessionId);

        //body
        BodyMessage bodyMessage = new DSBodyMessage(alerts);

        //new message
        NotificationMessage notificationMsg = new NotificationMessage();
        notificationMsg.setDigest(digestMessage);
        notificationMsg.setHeader(header);
        notificationMsg.setBody(bodyMessage);

        byte[] msg = notificationMsg.computeTriggerNotificationMessage();

        return new Message(Message.Type.STANDARD_1_2_NOTIFICATION_MESSAGE_TYPE, msg);
    }
}
