/**
 * Copyright (C) 2003-2007 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the Honest Public License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Honest Public License for more details.
 *
 * You should have received a copy of the Honest Public License
 * along with this program; if not, write to Funambol,
 * 643 Bair Island Road, Suite 305 - Redwood City, CA 94063, USA
 */

package com.funambol.server.notification.sender;

import com.funambol.framework.logging.FunambolLogger;
import com.funambol.framework.logging.FunambolLoggerFactory;
import com.funambol.framework.logging.Sync4jLoggerName;
import com.funambol.framework.notification.NotificationException;
import com.funambol.framework.tools.DbgTools;
import com.funambol.framework.tools.IOTools;

/**
 * This is a simple implementation that just writes the message in a file and/or
 * on the output console
 * @version $Id: SimpleWAPSenderImpl.java,v 1.6 2007/03/18 16:46:34 nichele Exp $
 */
public class SimpleWAPSenderImpl extends WAPSender {

    // ------------------------------------------------------------ Private data
    /**
     * The logger
     */
    private FunambolLogger log =
            FunambolLoggerFactory.getLogger(Sync4jLoggerName.SERVER_NOTIFICATION);

    // -------------------------------------------------------------- Properties
    /**
     * Should the messages be stored on files ?
     */
    private boolean logOnFiles    = false;

    public boolean isLogOnFiles() {
        return logOnFiles;
    }

    public void setLogOnFiles(boolean logOnFiles) {
        this.logOnFiles = logOnFiles;
    }

    // ---------------------------------------------------------- Public methods

    /**
     * Sends the notification message to a device.
     * @param deviceAddress address of the device where message should be sent
     * @param message notification message with wap header that has to be sent
     * @throws NotificationException
     */
    @Override
    public void sendMessage(String deviceAddress, byte[] message)
    throws NotificationException {

        if (log.isInfoEnabled()) {
            StringBuilder sb = new StringBuilder("Sending message '");
            sb.append(DbgTools.bytesToHex(message));
            sb.append("' to '").append(deviceAddress).append("'");

            log.info(sb.toString());
        }

        if (logOnFiles) {
            writeMessageOnFile(deviceAddress, message);
        }
    }

    // --------------------------------------------------------- Private methods
    /**
     * Writes an array of bytes into a file
     * @param deviceAddress the device address
     * @param message bytes to be written in file
     */
    private void writeMessageOnFile(String deviceAddress, byte[] message) {
        if (deviceAddress == null || deviceAddress.length() == 0) {
            deviceAddress = "Unknown";
        }
        try {
            IOTools.writeFile(message, deviceAddress + "_" + System.currentTimeMillis() + ".message");
        } catch (Exception ex) {
            // ignore this exception
        }
    }

}
