/**
 * Copyright (C) 2005-2007 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the Honest Public License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Honest Public License for more details.
 *
 * You should have received a copy of the Honest Public License
 * along with this program; if not, write to Funambol,
 * 643 Bair Island Road, Suite 305 - Redwood City, CA 94063, USA
 */
package com.funambol.server.notification.sender;

/**
 * This class represents a WDP header.
 *
 * @version $Id: WDPHeader.java,v 1.5 2007/01/18 10:08:38 nichele Exp $
 */
public class WDPHeader {

    // --------------------------------------------------------------- Constants
    public static final int DEFAULT_WAP_PUSH_DESTINATION_PORT = 2948;

    // ------------------------------------------------------------ Private data
    private byte[] header = null;

    public WDPHeader() {
        this(DEFAULT_WAP_PUSH_DESTINATION_PORT);
    }

    public WDPHeader(int destinationPort) {

        if (destinationPort > 65535) {
            throw new IllegalArgumentException("The destination port must be between 0 and 65535");
        }

        String sHex = Integer.toString(destinationPort, 16);
        int len = sHex.length();
        if (len == 1) {
            sHex = "000" + sHex;
        } else if (len == 2) {
            sHex = "00" + sHex;
        } else if (len == 3) {
            sHex = "0" + sHex;
        }
        byte destinationPortHigh = (byte)Integer.parseInt(sHex.substring(0, 2), 16);
        byte destinationPortLow =  (byte)Integer.parseInt(sHex.substring(2), 16);

        // User-Data-Header(UDHL) Length = 6 bytes
        byte length = 6;

        header = new byte[length + 1];

        header[0] = length;

        // UDH IE Identifier Port Number
        header[1] = (byte)0x05;

        // UDH port number IE length
        header[2] = (byte)0x04;

        // Destination port (high)
        header[3] = destinationPortHigh;

        // Destination port (low)
        header[4] = destinationPortLow;

        // Origination port (high)
        header[5] = (byte)0xC0;

        // Origination port (low)
        header[6] = (byte)0x02;

    }

    public WDPHeader(int destinationPort,
                     int originatorPort) {

        if (destinationPort > 65535) {
            throw new IllegalArgumentException("The destination port must be between 0 and 65535");
        }
        if (originatorPort > 65535) {
            throw new IllegalArgumentException("The destination port must be between 0 and 65535");
        }

        //
        // Handling destination port
        //
        String sHex = Integer.toString(destinationPort, 16);
        int len = sHex.length();
        if (len == 1) {
            sHex = "000" + sHex;
        } else if (len == 2) {
            sHex = "00" + sHex;
        } else if (len == 3) {
            sHex = "0" + sHex;
        }
        byte destinationPortHigh = (byte)Integer.parseInt(sHex.substring(0, 2), 16);
        byte destinationPortLow =  (byte)Integer.parseInt(sHex.substring(2), 16);

        //
        // Handling originator port
        //
        sHex = Integer.toString(originatorPort, 16);
        len = sHex.length();
        if (len == 1) {
            sHex = "000" + sHex;
        } else if (len == 2) {
            sHex = "00" + sHex;
        } else if (len == 3) {
            sHex = "0" + sHex;
        }
        byte originatorPortHigh = (byte)Integer.parseInt(sHex.substring(0, 2), 16);
        byte originatorPortLow =  (byte)Integer.parseInt(sHex.substring(2), 16);


        // User-Data-Header(UDHL) Length = 6 bytes
        byte length = 6;

        header = new byte[length + 1];

        header[0] = length;

        // UDH IE Identifier Port Number
        header[1] = (byte)0x05;

        // UDH port number IE length
        header[2] = (byte)0x04;

        // Destination port (high)
        header[3] = destinationPortHigh;

        // Destination port (low)
        header[4] = destinationPortLow;

        // Origination port (high)
        header[5] = originatorPortHigh;

        // Origination port (low)
        header[6] = originatorPortLow;

    }

    // ---------------------------------------------------------- Public Methods

    public byte[] getHeader() {
        return header;
    }
}
