/**
 * Copyright (C) 2003-2007 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the Honest Public License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Honest Public License for more details.
 *
 * You should have received a copy of the Honest Public License
 * along with this program; if not, write to Funambol,
 * 643 Bair Island Road, Suite 305 - Redwood City, CA 94063, USA
 */
package com.funambol.server.notification.sender;

/**
 * This class represents a WSP header.
 *
 * @version $Id: WSPHeader.java,v 1.6 2007/01/11 11:49:25 nichele Exp $
 */
public class WSPHeader {

    // --------------------------------------------------------------- Constants
    public static final byte CONTENT_TYPE_CODE_NOTIFICATION = (byte)0xCE;
    public static final byte CONTENT_TYPE_CODE_BOOTSTRAP    = (byte)0xC2;

    // ------------------------------------------------------------ Private data
    private byte[] header = null;

    private WSPHeader() {}

    public WSPHeader(byte contentTypeCode) {

        header = new byte[6];

        // Transaction ID / Push ID
        header[0] = (byte)0x01;

        // PDU Type(push)
        header[1] = (byte)0x06;

        // Headerslength (content type + headers)
        header[2] = (byte)0x03;

        // Content type
        header[3] = contentTypeCode;

        // X-WAP-Application-ID
        header[4] = (byte)0xAF;

        // Id for urn: x-wap-application:push.syncml
        header[5] = (byte)0x85;
    }
    
    public WSPHeader(byte[] contentType) {

        header = new byte[5 + contentType.length];

        // Transaction ID / Push ID
        header[0] = (byte)0x01;

        // PDU Type(push)
        header[1] = (byte)0x06;

        // Headerslength (content type + headers)
        header[2] = (byte)(2 + contentType.length);

        // Content type
        System.arraycopy(contentType, 0, header, 3, contentType.length);

        // X-WAP-Application-ID
        header[3 + contentType.length ] = (byte)0xAF;

        // Id for urn: x-wap-application:push.syncml
        header[4 + contentType.length] = (byte)0x85;
    }    

    // ---------------------------------------------------------- Public Methods

    public byte[] getHeader() {
        return header;
    }
}
