/**
 * Copyright (C) 2006-2007 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the Honest Public License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Honest Public License for more details.
 *
 * You should have received a copy of the Honest Public License
 * along with this program; if not, write to Funambol,
 * 643 Bair Island Road, Suite 305 - Redwood City, CA 94063, USA
 */

package com.funambol.server.tools.directorymonitor;

import java.io.File;

import org.apache.commons.lang.builder.ToStringBuilder;

/**
 * It's an event used to notify changes in a file.
 *
 * @version $Id: FileChangeEvent.java,v 1.3 2007/01/11 11:49:25 nichele Exp $
 */
public class FileChangeEvent {

    // --------------------------------------------------------------- Constants

    /** The event type */
    public static enum EventType {FILE_ADDED,
                                  FILE_CHANGED,
                                  FILE_DELETED};

    // -------------------------------------------------------------- Properties

    /**
     * The file that the event is relative to
     */
    private File file = null;

    /**
     * the file added/deleted/updated
     * @return the file added/deleted/updated
     */
    public File getFile() {
        return file;
    }

    /**
     * The scan timestamp the event is relative to
     */
    private long scanTimeStamp = 0;

    public long getScanTimeStamp() {
        return scanTimeStamp;
    }

    /**
     *  The type of the event
     */
    private EventType eventType = null;


    /**
     * Returns the type of the event. See {@link EventType}
     * @return the type of the event
     */
    public EventType getEventType() {
        return eventType;
    }


    // ------------------------------------------------------------- Constructor

    /**
     * Creates a new FileChangeEvent with the given file and eventType
     * @param scanTimeStamp the scan timestamp the event is relative to
     * @param file the file that the event is relative to
     * @param eventType the type of the event
     */
    public FileChangeEvent(long      scanTimeStamp,
                           File      file,
                           EventType eventType) {

        this.scanTimeStamp = scanTimeStamp;
        this.file = file;
        this.eventType = eventType;
    }


    /**
     * Returns a string representation of this event
     * @return a string representation of this event
     */
    public String toString() {
        ToStringBuilder sb = new ToStringBuilder(this);

        sb.append("scanTimeStamp"    , scanTimeStamp);
        sb.append("file"             , file);
        sb.append("eventType"        , eventType);

        return sb.toString();
    }
}
