/**
 * Copyright (C) 2007 Funambol
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the Honest Public License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * Honest Public License for more details.
 *
 * You should have received a copy of the Honest Public License
 * along with this program; if not, write to Funambol,
 * 643 Bair Island Road, Suite 305 - Redwood City, CA 94063, USA
 */

package com.funambol.server.tools.id;

import java.util.HashMap;
import java.util.Map;

import javax.sql.DataSource;

/**
 * Provides a factory of DBIDGenerators
 *
 * @version $Id: DBIDGeneratorFactory.java,v 1.1 2007/01/18 19:32:20 nichele Exp $
 */
public class DBIDGeneratorFactory {

    // ------------------------------------------------------------ Private Data

    private static Map<String, DBIDGenerator> instancesCache = new HashMap();


    /**
     * Returns a new instance of DBIDGenerator searching for already created instance
     * in the instancesCache. If no instance is found, a new one is created using
     * the given datasouce. Using this factory, a class that needs a DBIDGenerator,
     * can use a not-static instance because anyhow, calling
     * DBIDGeneratorFactory.getDBIDGenerator with the same nameSpace, always the same
     * instance is returned.
     * <p>
     * <b>NOTE: if an instance of a DBIDGenerator with the same nameSpace has been
     * already created, this is returned without using the given DataSource. So,
     * the use of the given datasource is not guaranteed.
     * This is accettable in a lot of cases, because usually a server application uses
     * just one datasource (obtained via jndi) or at least different datasources
     * that use the same database. <br/>As alternative, create yourself a <code>
     * com.funambol.server.tools.id.DBIDGenerator</code> with your datasource.</b>
     * @param name the name of the needed space
     * @param dataSource the datasource to use creating the DBIDGenerators
     * @return DBIDGenerator the DBIDGenerator with the given name space
     */
    public static synchronized DBIDGenerator getDBIDGenerator(String     nameSpace,
                                                              DataSource dataSource) {

        DBIDGenerator instance = instancesCache.get(nameSpace);
        if (instance == null) {
            instance = new DBIDGenerator(dataSource, nameSpace);
            instancesCache.put(nameSpace, instance);
        }
        return instance;
    }
}
